;nyquist plug-in
;version 3
;type analyze
;name "Tempo Teller"
;action "Analyzing..."
;; rjh-tempo-teller.ny by Robert Haenggi, February 2014 
;; Released under terms of the GNU General Public License version 2:
;; http://www.gnu.org/licenses/old-licenses/gpl-2.0.html 
;; Plug-in version 2.1 
;; requires Audacity 1.3.12 or later, developed under Audacity 2.0.6 Alpha
;;
;;control dummy "oops" int "" 0 -1 1
;;
;; Slope replacement
(defun differentiator (snd amount)
  (snd-biquad snd 1 (- amount)  0 0 0 0 0))  
;;
;; frequency interpolation
(defun quadratic (y-1 y0 y1)
  (let ((pos (/ (- Y1  Y-1)(* 2.0 (- (* 2 Y0) Y1 Y-1)))))
       ; return position, estimated peak and half curve
       (list pos (- y0 (* 0.25 (* pos (- y-1 y1))))  
             (* 0.5 (+ (- y-1 (* 2 y0)) y1)))))
;;
;; rounding
(defun roundto (val quan) (/ (round (* quan val)) (float quan)))
;;
;; Magnitude Spectrum
(defun fft-mag (frame)
  (let* ((n (length frame))
         (magnitude (make-array (/ n 2))))
        (dotimes (i (- (length magnitude) 2))
           (setf (aref magnitude (1+ i)) 
              (sqrt (+  (expt (/ (aref frame (1+ (* i 2))) (/ n 2)) 2.0)
                 (expt (/ (aref frame (* (1+ i) 2)) (/ n 2)) 2.0)))))
        (setf (aref magnitude 0) (/ (aref frame 0) n))
        (setf (aref magnitude (1- (/ n 2))) (/ (aref frame (1- n)) n))
        magnitude ))
;;
;; Harmonic Product Spectrum (or sum)
(defun hps (spectrum partials function)
  (let*        ((*sr* (length spectrum))
       (s-mag (snd-from-array 0 *sr* spectrum))
       (noharm s-mag)
       (s-mag   (dotimes  (i partials noharm) (setf noharm (apply function (list noharm (scale-srate 
          (snd-down (/ *sr* (+ 2 i)) s-mag) (+ 2 i)))))))) 
        (snd-samples s-mag ny:all)))
        ;;
;; Find true peak and frequency by interpolating bins
(defun correct (input &aux (output '()))
  (do* ((i 30 (1+ i)) 
                (y1 (nth (1- i) input) (nth (1- i) input)) 
                (y2 (nth i input) (nth i input))
                (y3 (nth (1+ i) input) (nth (1+ i) input)))
             ((= i (min 600 (1- (length input)))) output)
     (cond
        ((and (> (car y2) (car y1)) (> (car y2) (car y3))
         (setf quadratic (quadratic (car y1) (car y2) (car y3)))
         (push (list (cadr quadratic) (+ (cadr y2)  (car quadratic))) output)))
        (t nil))))
(defun phase (frame index)
  (let* ((i2 (* 2 index)) (r2 (1- i2)) 
         (r3 (1+ i2)) (i3 (1+ r3))
         (i1 (1- r2)) (r1 (1- i1))
(real (aref frame r2))
(imaginary  (aref frame i2))
(phi (* 180  (/ (atan real imaginary) pi)))  
)
(print phi)))
;;;; Main
;;;;Preparing audio
(setf s2 (resample (extract-abs 0 30
       (if (soundp s) s (scale 0.5 (sim (aref s 0) (aref s 1))))) 51200))
;; We could simply get the highest peak, but we filter 
;; the Spectrum firstly a little bit 
"Measurement has failed"
(let* ((flist '())
       (sig (snd-avg s2 750 750 op-peak))
       (*sr* (snd-srate sig))
       (sig (control-srate-abs *sr* (s-max 0 (differentiator sig 0.8))))
       (*fr* 4096)
       (single-frame (snd-fft (snd-copy sig) *fr* *fr* nil))
       (mag (fft-mag single-frame))
       (mag (hps mag 5 'sim))
       (flist 
          (do* ((i 0   (1+ i))) ((= i  (length mag)) (reverse flist))
                (push (list (max -1e15 (linear-to-db (aref mag i))) i) flist)))
       (flist (correct (remove "" flist))) 
       (flist (sort flist #'(lambda (a1 a2) (< (car a1) (car a2) ))))
       ;(threshold  (- (car (car (last flist))) 28))
       ;(flist (remove-if #'(lambda (a) (< (car a) threshold)) flist))
       (tempo (cadr (car (last flist))))
       (phase (phase single-frame (round tempo))))
     (format nil "~a bpm~%~a Degrees" (roundto tempo 100) phase))